'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.getFormStateKey = getFormStateKey;

var _get = require('../utils/get');

var _get2 = _interopRequireDefault(_get);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _pathStartsWith = require('../utils/path-starts-with');

var _pathStartsWith2 = _interopRequireDefault(_pathStartsWith);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function joinPaths() {
  for (var _len = arguments.length, paths = Array(_len), _key = 0; _key < _len; _key++) {
    paths[_key] = arguments[_key];
  }

  return paths.filter(function (path) {
    return !!path && path.length;
  }).join('.');
}

function getFormStateKey(state, model) {
  var currentPath = arguments.length <= 2 || arguments[2] === undefined ? '' : arguments[2];

  var deepCandidateKeys = [];
  var result = null;

  Object.keys(state).some(function (key) {
    var subState = state[key];

    if (subState && subState.$form) {
      if ((0, _pathStartsWith2.default)(model, subState.$form.model) || subState.$form.model === '') {
        var _ret = function () {
          var localPath = (0, _pathStartsWith.pathDifference)(model, subState.$form.model);

          var resultPath = [currentPath, key];
          var currentState = subState;

          localPath.every(function (segment) {
            if (currentState[segment] && currentState[segment].$form) {
              currentState = currentState[segment];
              resultPath.push(segment);

              return true;
            }

            return false;
          });

          result = joinPaths.apply(undefined, resultPath);

          return {
            v: true
          };
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
      }

      return false;
    }

    if ((0, _isPlainObject2.default)(subState)) {
      deepCandidateKeys.push(key);
    }

    return false;
  });

  if (result) return result;

  deepCandidateKeys.some(function (key) {
    result = getFormStateKey(state[key], model, joinPaths(currentPath, key));

    return !!result;
  });

  if (result) return result;

  return null;
}

var formStateKeyCache = {};

var getFormStateKeyCached = function () {
  return function (state, modelString) {
    if (formStateKeyCache[modelString]) return formStateKeyCache[modelString];

    var result = getFormStateKey(state, modelString);

    formStateKeyCache[modelString] = result; // eslint-disable-line no-return-assign

    return result;
  };
}();

function getForm(state, modelString) {
  var get = arguments.length <= 2 || arguments[2] === undefined ? _get2.default : arguments[2];

  var formStateKey = getFormStateKeyCached(state, modelString);

  if (!formStateKey) {
    return null;
  }

  return get(state, formStateKey);
}

getForm.clearCache = function () {
  return formStateKeyCache = {};
}; // eslint-disable-line no-return-assign

exports.default = getForm;